var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

import { value, onCreated, onBeforeDestroy } from "vue-function-api";

/**
 * Async helper function that returns three reactive values:
 * * `isLoading`, a boolean that is true during pending state;
 * * `data`, contains the resolved value in the fulfilled state; and
 * * `error`, contains the exception in the rejected state.
 *
 * @param {function} promiseFn A function that returns a Promise.
 * @param {object} params Parameters passed as first argument to the promise function.
 * @returns {object} Object literal containing `isLoading`, `error` and `data` value wrappers.
 */
export function useAsync(promiseFn, params) {
  const isLoading = value(true);
  const error = value(null);
  const data = value(null);
  let controller = null;

  onCreated(_asyncToGenerator(function* () {
    try {
      controller = new AbortController();
      const { signal } = controller;
      const result = yield promiseFn(params, signal);
      data.value = result;
    } catch (e) {
      error.value = e;
    } finally {
      isLoading.value = false;
    }
  }));

  onBeforeDestroy(() => {
    if (controller !== null) {
      controller.abort();
    }
  });

  return {
    isLoading,
    error,
    data
  };
}

/**
 * Fetch helper function that accepts the same arguments as `fetch` and returns the same reactive values as `useAsync`.
 * If the `Accept` header is set to `application/json` in the `requestInit` object, the response will be parsed as JSON,
 * else text.
 *
 * @param {string|object} requestInfo URL or request object.
 * @param {object} requestInit Init parameters for the request.
 * @returns {object} Object literal containing `isLoading`, `error` and `data` value wrappers.
 */
export function useFetch(requestInfo, requestInit = {}) {
  let doFetch = (() => {
    var _ref2 = _asyncToGenerator(function* (params, signal) {
      const res = yield fetch(requestInfo, _extends({}, requestInit, {
        signal
      }));
      if (!res.ok) {
        throw new Error(res);
      }
      if (requestInit.headers && requestInit.headers.Accept === "application/json") {
        return res.json();
      }
      return res.text();
    });

    return function doFetch(_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  })();

  return useAsync(doFetch);
}
//# sourceMappingURL=index.js.map